@@ -71,11 +71,11 @@ struct QTokenizerPrivate {
     {
     }
 
-    bool isDelimiter(char_type c) const {
+    [[nodiscard]] bool isDelimiter(char_type c) const {
         return delimiters.contains(c);
     }
 
-    bool isQuote(char_type c) const {
+    [[nodiscard]] bool isQuote(char_type c) const {
         return quotes.contains(c);
     }
 
@@ -212,14 +212,14 @@ public:
        Returns true if the current token is a delimiter,
        if one more more delimiting characters have been set.
      */
-    bool isDelimiter() const { return d->isDelim; }
+    [[nodiscard]] bool isDelimiter() const { return d->isDelim; }
 
     /*!
        Returns the current token.
 
        Use \c hasNext() to fetch the next token.
      */
-    T next() const {
+    [[nodiscard]] T next() const {
         int len = std::distance(d->tokenBegin, d->tokenEnd);
         const_iterator tmpStart = d->tokenBegin;
         if (!d->returnQuotes && len > 1 && d->isQuote(*d->tokenBegin)) {
