@@ -27,6 +27,7 @@
 #include <memory>
 
 class QSettings;
+class FakeAccountState;
 
 namespace OCC {
 
@@ -182,10 +183,10 @@ public:
 public slots:
     /// Triggers a ping to the server to update state and
     /// connection status and errors.
-    void checkConnectivity();
+    virtual void checkConnectivity();
 
 private:
-    void setState(State state);
+    virtual void setState(State state);
     void fetchNavigationApps();
 
     int retryCount() const;
@@ -193,21 +194,21 @@ private:
     void resetRetryCount();
 
 signals:
-    void stateChanged(State state);
+    void stateChanged(OCC::AccountState::State state);
     void isConnectedChanged();
     void hasFetchedNavigationApps();
     void statusChanged();
     void desktopNotificationsAllowedChanged();
 
 protected Q_SLOTS:
-    void slotConnectionValidatorResult(ConnectionValidator::Status status, const QStringList &errors);
+    void slotConnectionValidatorResult(OCC::ConnectionValidator::Status status, const QStringList &errors);
 
     /// When client gets a 401 or 403 checks if server requested remote wipe
     /// before asking for user credentials again
     void slotHandleRemoteWipeCheck();
 
-    void slotCredentialsFetched(AbstractCredentials *creds);
-    void slotCredentialsAsked(AbstractCredentials *creds);
+    void slotCredentialsFetched(OCC::AbstractCredentials *creds);
+    void slotCredentialsAsked(OCC::AbstractCredentials *creds);
 
     void slotNavigationAppsFetched(const QJsonDocument &reply, int statusCode);
     void slotEtagResponseHeaderReceived(const QByteArray &value, int statusCode);
@@ -261,6 +262,9 @@ private:
     QTimer _checkConnectionTimer;
     QElapsedTimer _lastCheckConnectionTimer;
 
+    explicit AccountState() = default;
+
+    friend class ::FakeAccountState;
 };
 
 class AccountApp : public QObject
@@ -271,10 +275,10 @@ public:
         const QString &id, const QUrl &iconUrl,
         QObject* parent = nullptr);
 
-    QString name() const;
-    QUrl url() const;
-    QString id() const;
-    QUrl iconUrl() const;
+    [[nodiscard]] QString name() const;
+    [[nodiscard]] QUrl url() const;
+    [[nodiscard]] QString id() const;
+    [[nodiscard]] QUrl iconUrl() const;
 
 private:
     QString _name;
