@@ -64,13 +64,13 @@ public:
     void showHelp();
     void showHint(std::string errorHint);
     bool debugMode();
-    bool backgroundMode() const;
+    [[nodiscard]] bool backgroundMode() const;
     bool versionOnly(); // only display the version?
     void showVersion();
 
     void showMainDialog();
 
-    ownCloudGui *gui() const;
+    [[nodiscard]] ownCloudGui *gui() const;
 
     bool event(QEvent *event) override;
 
@@ -87,8 +87,6 @@ public slots:
     /// Attempt to show() the tray icon again. Used if no systray was available initially.
     void tryTrayAgain();
 
-    void handleEditLocally(const QUrl &url) const;
-
 protected:
     void parseOptions(const QStringList &);
     void setupTranslations();
@@ -96,7 +94,7 @@ protected:
 
 signals:
     void folderRemoved();
-    void folderStateChanged(Folder *);
+    void folderStateChanged(OCC::Folder *);
     void isShowingSettingsDialog();
 
 protected slots:
@@ -104,8 +102,8 @@ protected slots:
     void slotCheckConnection();
     void slotUseMonoIconsChanged(bool);
     void slotCleanup();
-    void slotAccountStateAdded(AccountState *accountState);
-    void slotAccountStateRemoved(AccountState *accountState);
+    void slotAccountStateAdded(OCC::AccountState *accountState);
+    void slotAccountStateRemoved(OCC::AccountState *accountState);
     void slotSystemOnlineConfigurationChanged(QNetworkConfiguration);
     void slotGuiIsShowingSettings();
 
@@ -147,6 +145,9 @@ private:
     QNetworkConfigurationManager _networkConfigurationManager;
     QTimer _checkConnectionTimer;
 
+    QString _overrideServerUrl;
+    QString _overrideLocalDir;
+
 #if defined(WITH_CRASHREPORTER)
     QScopedPointer<CrashReporter::Handler> _crashHandler;
 #endif
