@@ -12,8 +12,9 @@ class QVBoxLayout;
 
 namespace OCC {
 
-class HeaderBanner;
+#ifdef WITH_WEBENGINE
 class WebView;
+#endif // WITH_WEBENGINE
 class Flow2AuthWidget;
 
 class WebFlowCredentialsDialog : public QDialog
@@ -26,7 +27,7 @@ public:
     void setInfo(const QString &msg);
     void setError(const QString &error);
 
-    bool isUsingFlow2() const {
+    [[nodiscard]] bool isUsingFlow2() const {
         return _useFlow2;
     }
 
@@ -50,13 +51,14 @@ private:
     bool _useFlow2;
 
     Flow2AuthWidget *_flow2AuthWidget;
+#ifdef WITH_WEBENGINE
     WebView *_webView;
+#endif // WITH_WEBENGINE
 
     QLabel *_errorLabel;
     QLabel *_infoLabel;
     QVBoxLayout *_layout;
     QVBoxLayout *_containerLayout;
-    HeaderBanner *_headerBanner;
 };
 
 } // namespace OCC
