@@ -40,7 +40,7 @@ void OcsJob::setVerb(const QByteArray &verb)
 
 void OcsJob::addParam(const QString &name, const QString &value)
 {
-    _params.append(qMakePair(name, value));
+    _params.insert(name, value);
 }
 
 void OcsJob::addPassStatusCode(int code)
@@ -58,16 +58,21 @@ void OcsJob::addRawHeader(const QByteArray &headerName, const QByteArray &value)
     _request.setRawHeader(headerName, value);
 }
 
+QString OcsJob::getParamValue(const QString &key) const
+{
+    return _params.value(key);
+}
+
 static QUrlQuery percentEncodeQueryItems(
-    const QList<QPair<QString, QString>> &items)
+    const QHash<QString, QString> &items)
 {
     QUrlQuery result;
     // Note: QUrlQuery::setQueryItems() does not fully percent encode
     // the query items, see #5042
-    foreach (const auto &item, items) {
+    for (auto it = std::cbegin(items); it != std::cend(items); ++it) {
         result.addQueryItem(
-            QUrl::toPercentEncoding(item.first),
-            QUrl::toPercentEncoding(item.second));
+            QUrl::toPercentEncoding(it.key()),
+            QUrl::toPercentEncoding(it.value()));
     }
     return result;
 }
@@ -85,13 +90,13 @@ void OcsJob::start()
     } else if (_verb == "POST" || _verb == "PUT") {
         // Url encode the _postParams and put them in a buffer.
         QByteArray postData;
-        Q_FOREACH (auto tmp, _params) {
+        for (auto it = std::cbegin(_params); it != std::cend(_params); ++it) {
             if (!postData.isEmpty()) {
                 postData.append("&");
             }
-            postData.append(QUrl::toPercentEncoding(tmp.first));
+            postData.append(QUrl::toPercentEncoding(it.key()));
             postData.append("=");
-            postData.append(QUrl::toPercentEncoding(tmp.second));
+            postData.append(QUrl::toPercentEncoding(it.value()));
         }
         buffer->setData(postData);
     }
