@@ -19,7 +19,7 @@ signals:
     /**
      * Notify if wipe was requested
      */
-    void authorized(AccountState*);
+    void authorized(OCC::AccountState*);
 
     /**
      * Notify if user only needs to login again
@@ -44,7 +44,7 @@ private slots:
      * Once the client has wiped all the required data a POST to
      * <server>/index.php/core/wipe/success
      */
-    void notifyServerSuccessJob(AccountState *accountState, bool);
+    void notifyServerSuccessJob(OCC::AccountState *accountState, bool);
     void notifyServerSuccessJobSlot();
 
 private:
@@ -58,4 +58,4 @@ private:
     friend class ::TestRemoteWipe;
 };
 }
-#endif // REMOTEWIPE_H
\ No newline at end of file
+#endif // REMOTEWIPE_H
