@@ -15,10 +15,9 @@
 #ifndef SOCKETAPI_H
 #define SOCKETAPI_H
 
-#include "syncfileitem.h"
 #include "common/syncfilestatus.h"
-#include "sharedialog.h" // for the ShareDialogStartPage
 #include "common/syncjournalfilerecord.h"
+#include "syncfileitem.h"
 
 #include "config.h"
 
@@ -29,8 +28,8 @@ class QLocalSocket;
 class QStringList;
 class QFileInfo;
 
-namespace OCC {
-
+namespace OCC
+{
 class SyncFileStatus;
 class Folder;
 class SocketListener;
@@ -57,14 +56,14 @@ public:
     ~SocketApi() override;
 
 public slots:
-    void slotUpdateFolderView(Folder *f);
+    void slotUpdateFolderView(OCC::Folder *f);
     void slotUnregisterPath(const QString &alias);
     void slotRegisterPath(const QString &alias);
-    void broadcastStatusPushMessage(const QString &systemPath, SyncFileStatus fileStatus);
+    void broadcastStatusPushMessage(const QString &systemPath, OCC::SyncFileStatus fileStatus);
 
 signals:
-    void shareCommandReceived(const QString &sharePath, const QString &localPath, ShareDialogStartPage startPage);
-    void fileActivityCommandReceived(const QString &objectName, const int objectId);
+    void shareCommandReceived(const QString &localPath);
+    void fileActivityCommandReceived(const QString &localPath);
 
 private slots:
     void slotNewConnection();
@@ -83,12 +82,12 @@ private:
     struct FileData
     {
         static FileData get(const QString &localFile);
-        SyncFileStatus syncFileStatus() const;
-        SyncJournalFileRecord journalRecord() const;
-        FileData parentFolder() const;
+        [[nodiscard]] SyncFileStatus syncFileStatus() const;
+        [[nodiscard]] SyncJournalFileRecord journalRecord() const;
+        [[nodiscard]] FileData parentFolder() const;
 
         // Relative path of the file locally, without any vfs suffix
-        QString folderRelativePathNoVfsSuffix() const;
+        [[nodiscard]] QString folderRelativePathNoVfsSuffix() const;
 
         Folder *folder;
         // Absolute path of the file locally. (May be a virtual file)
@@ -102,63 +101,72 @@ private:
     void broadcastMessage(const QString &msg, bool doWait = false);
 
     // opens share dialog, sends reply
-    void processShareRequest(const QString &localFile, SocketListener *listener, ShareDialogStartPage startPage);
+    void processShareRequest(const QString &localFile, SocketListener *listener);
+    void processLeaveShareRequest(const QString &localFile, SocketListener *listener);
     void processFileActivityRequest(const QString &localFile);
+    void processEncryptRequest(const QString &localFile);
 
-    Q_INVOKABLE void command_RETRIEVE_FOLDER_STATUS(const QString &argument, SocketListener *listener);
-    Q_INVOKABLE void command_RETRIEVE_FILE_STATUS(const QString &argument, SocketListener *listener);
+    Q_INVOKABLE void command_RETRIEVE_FOLDER_STATUS(const QString &argument, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_RETRIEVE_FILE_STATUS(const QString &argument, OCC::SocketListener *listener);
 
-    Q_INVOKABLE void command_VERSION(const QString &argument, SocketListener *listener);
+    Q_INVOKABLE void command_VERSION(const QString &argument, OCC::SocketListener *listener);
 
-    Q_INVOKABLE void command_SHARE_MENU_TITLE(const QString &argument, SocketListener *listener);
+    Q_INVOKABLE void command_SHARE_MENU_TITLE(const QString &argument, OCC::SocketListener *listener);
 
     // The context menu actions
-    Q_INVOKABLE void command_ACTIVITY(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_SHARE(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_MANAGE_PUBLIC_LINKS(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_COPY_PUBLIC_LINK(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_COPY_PRIVATE_LINK(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_EMAIL_PRIVATE_LINK(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_OPEN_PRIVATE_LINK(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_MAKE_AVAILABLE_LOCALLY(const QString &filesArg, SocketListener *listener);
-    Q_INVOKABLE void command_MAKE_ONLINE_ONLY(const QString &filesArg, SocketListener *listener);
-    Q_INVOKABLE void command_RESOLVE_CONFLICT(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_DELETE_ITEM(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_MOVE_ITEM(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_LOCK_FILE(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_UNLOCK_FILE(const QString &localFile, SocketListener *listener);
+    Q_INVOKABLE void command_ACTIVITY(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_ENCRYPT(const QString &localFile, SocketListener *listener);
+    Q_INVOKABLE void command_SHARE(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_LEAVESHARE(const QString &localFile, SocketListener *listener);
+    Q_INVOKABLE void command_MANAGE_PUBLIC_LINKS(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_COPY_PUBLIC_LINK(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_COPY_PRIVATE_LINK(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_EMAIL_PRIVATE_LINK(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_OPEN_PRIVATE_LINK(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_MAKE_AVAILABLE_LOCALLY(const QString &filesArg, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_MAKE_ONLINE_ONLY(const QString &filesArg, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_RESOLVE_CONFLICT(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_DELETE_ITEM(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_MOVE_ITEM(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_LOCK_FILE(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_UNLOCK_FILE(const QString &localFile, OCC::SocketListener *listener);
 
     void setFileLock(const QString &localFile, const SyncFileItem::LockStatus lockState) const;
 
     // Windows Shell / Explorer pinning fallbacks, see issue: https://github.com/nextcloud/desktop/issues/1599
 #ifdef Q_OS_WIN
-    Q_INVOKABLE void command_COPYASPATH(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_OPENNEWWINDOW(const QString &localFile, SocketListener *listener);
-    Q_INVOKABLE void command_OPEN(const QString &localFile, SocketListener *listener);
+    Q_INVOKABLE void command_COPYASPATH(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_OPENNEWWINDOW(const QString &localFile, OCC::SocketListener *listener);
+    Q_INVOKABLE void command_OPEN(const QString &localFile, OCC::SocketListener *listener);
 #endif
 
     // External sync
-    Q_INVOKABLE void command_V2_LIST_ACCOUNTS(const QSharedPointer<SocketApiJobV2> &job) const;
-    Q_INVOKABLE void command_V2_UPLOAD_FILES_FROM(const QSharedPointer<SocketApiJobV2> &job) const;
+    Q_INVOKABLE void command_V2_LIST_ACCOUNTS(const QSharedPointer<OCC::SocketApiJobV2> &job) const;
+    Q_INVOKABLE void command_V2_UPLOAD_FILES_FROM(const QSharedPointer<OCC::SocketApiJobV2> &job) const;
 
     // Fetch the private link and call targetFun
     void fetchPrivateLinkUrlHelper(const QString &localFile, const std::function<void(const QString &url)> &targetFun);
 
     /** Sends translated/branded strings that may be useful to the integration */
-    Q_INVOKABLE void command_GET_STRINGS(const QString &argument, SocketListener *listener);
+    Q_INVOKABLE void command_GET_STRINGS(const QString &argument, OCC::SocketListener *listener);
 
     // Sends the context menu options relating to sharing to listener
     void sendSharingContextMenuOptions(const FileData &fileData, SocketListener *listener, bool enabled);
 
+    void sendEncryptFolderCommandMenuEntries(const QFileInfo &fileInfo,
+                                             const FileData &fileData,
+                                             const bool isE2eEncryptedPath,
+                                             const OCC::SocketListener* const listener) const;
+
     void sendLockFileCommandMenuEntries(const QFileInfo &fileInfo,
-                                        Folder * const syncFolder,
+                                        Folder *const syncFolder,
                                         const FileData &fileData,
-                                        const SocketListener * const listener) const;
+                                        const SocketListener *const listener) const;
 
     void sendLockFileInfoMenuEntries(const QFileInfo &fileInfo,
-                                     Folder * const syncFolder,
+                                     Folder* const syncFolder,
                                      const FileData &fileData,
-                                     const SocketListener * const listener,
+                                     const SocketListener* const listener,
                                      const SyncJournalFileRecord &record) const;
 
     /** Send the list of menu item. (added in version 1.1)
@@ -168,10 +176,10 @@ private:
      * If flag contains 'd', the menu should be disabled
      * and ends with GET_MENU_ITEMS:END
      */
-    Q_INVOKABLE void command_GET_MENU_ITEMS(const QString &argument, SocketListener *listener);
+    Q_INVOKABLE void command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListener *listener);
 
     /// Direct Editing
-    Q_INVOKABLE void command_EDIT(const QString &localFile, SocketListener *listener);
+    Q_INVOKABLE void command_EDIT(const QString &localFile, OCC::SocketListener *listener);
     DirectEditor* getDirectEditorForLocalFile(const QString &localFile);
 
 #if GUI_TESTING
