@@ -43,16 +43,16 @@ class OwncloudSetupPage : public QWizardPage
     Q_OBJECT
 public:
     OwncloudSetupPage(QWidget *parent = nullptr);
-    ~OwncloudSetupPage();
+    ~OwncloudSetupPage() override;
 
-    bool isComplete() const override;
+    [[nodiscard]] bool isComplete() const override;
     void initializePage() override;
-    int nextId() const override;
+    [[nodiscard]] int nextId() const override;
     void setServerUrl(const QString &);
     void setAllowPasswordStorage(bool);
     bool validatePage() override;
-    QString url() const;
-    QString localFolder() const;
+    [[nodiscard]] QString url() const;
+    [[nodiscard]] QString localFolder() const;
     void setRemoteFolder(const QString &remoteFolder);
     void setMultipleFoldersExist(bool exist);
     void setAuthType(DetermineAuthTypeJob::AuthType type);
@@ -67,10 +67,6 @@ public slots:
 protected slots:
     void slotUrlChanged(const QString &);
     void slotUrlEditFinished();
-#ifdef WITH_PROVIDERS
-    void slotLogin();
-    void slotGotoProviderList();
-#endif
 
     void setupCustomization();
 
