@@ -24,6 +24,7 @@
 #include <QDateTime>
 #include <QTimer>
 #include "accountfwd.h"
+#include "common/asserts.h"
 
 class QUrl;
 
@@ -40,20 +41,20 @@ class OWNCLOUDSYNC_EXPORT AbstractNetworkJob : public QObject
     Q_OBJECT
 public:
     explicit AbstractNetworkJob(AccountPtr account, const QString &path, QObject *parent = nullptr);
-    virtual ~AbstractNetworkJob();
+    ~AbstractNetworkJob() override;
 
     virtual void start();
 
-    AccountPtr account() const { return _account; }
+    [[nodiscard]] AccountPtr account() const { return _account; }
 
     void setPath(const QString &path);
-    QString path() const { return _path; }
+    [[nodiscard]] QString path() const { return _path; }
 
     void setReply(QNetworkReply *reply);
-    QNetworkReply *reply() const { return _reply; }
+    [[nodiscard]] QNetworkReply *reply() const { return _reply; }
 
     void setIgnoreCredentialFailure(bool ignore);
-    bool ignoreCredentialFailure() const { return _ignoreCredentialFailure; }
+    [[nodiscard]] bool ignoreCredentialFailure() const { return _ignoreCredentialFailure; }
 
     /** Whether to handle redirects transparently.
      *
@@ -66,15 +67,17 @@ public:
      * requests where custom handling is necessary.
      */
     void setFollowRedirects(bool follow);
-    bool followRedirects() const { return _followRedirects; }
+    [[nodiscard]] bool followRedirects() const { return _followRedirects; }
 
     QByteArray responseTimestamp();
+    /* Content of the X-Request-ID header. (Only set after the request is sent) */
+    QByteArray requestId();
 
-    qint64 timeoutMsec() const { return _timer.interval(); }
-    bool timedOut() const { return _timedout; }
+    [[nodiscard]] qint64 timeoutMsec() const { return _timer.interval(); }
+    [[nodiscard]] bool timedOut() const { return _timedout; }
 
     /** Returns an error message, if any. */
-    QString errorString() const;
+    [[nodiscard]] virtual QString errorString() const;
 
     /** Like errorString, but also checking the reply body for information.
      *
@@ -88,6 +91,9 @@ public:
      */
     QString errorStringParsingBody(QByteArray *body = nullptr);
 
+    /** Make a new request */
+    void retry();
+
     /** static variable the HTTP timeout (in seconds). If set to 0, the default will be used
      */
     static int httpTimeout;
@@ -122,6 +128,9 @@ protected:
         QNetworkRequest req = QNetworkRequest(),
         QIODevice *requestBody = nullptr);
 
+    QNetworkReply *sendRequest(const QByteArray &verb, const QUrl &url,
+        QNetworkRequest req, const QByteArray &requestBody);
+
     // sendRequest does not take a relative path instead of an url,
     // but the old API allowed that. We have this undefined overload
     // to help catch usage errors
@@ -129,6 +138,9 @@ protected:
         QNetworkRequest req = QNetworkRequest(),
         QIODevice *requestBody = nullptr);
 
+    QNetworkReply *sendRequest(const QByteArray &verb, const QUrl &url,
+        QNetworkRequest req, QHttpMultiPart *requestBody);
+
     /** Makes this job drive a pre-made QNetworkReply
      *
      * This reply cannot have a QIODevice request body because we can't get
@@ -147,12 +159,12 @@ protected:
     virtual void newReplyHook(QNetworkReply *) {}
 
     /// Creates a url for the account from a relative path
-    QUrl makeAccountUrl(const QString &relativePath) const;
+    [[nodiscard]] QUrl makeAccountUrl(const QString &relativePath) const;
 
     /// Like makeAccountUrl() but uses the account's dav base path
-    QUrl makeDavUrl(const QString &relativePath) const;
+    [[nodiscard]] QUrl makeDavUrl(const QString &relativePath) const;
 
-    int maxRedirects() const { return 10; }
+    [[nodiscard]] int maxRedirects() const { return 10; }
 
     /** Called at the end of QNetworkReply::finished processing.
      *
@@ -189,9 +201,7 @@ private:
     QString _path;
     QTimer _timer;
     int _redirectCount = 0;
-#if (QT_VERSION >= 0x050800)
     int _http2ResendCount = 0;
-#endif
 
     // Set by the xyzRequest() functions and needed to be able to redirect
     // requests, should it be required.
@@ -226,12 +236,6 @@ QString OWNCLOUDSYNC_EXPORT extractErrorMessage(const QByteArray &errorResponse)
 /** Builds a error message based on the error and the reply body. */
 QString OWNCLOUDSYNC_EXPORT errorMessage(const QString &baseError, const QByteArray &body);
 
-/** Helper to construct the HTTP verb used in the request
- *
- * Returns an empty QByteArray for UnknownOperation.
- */
-QByteArray OWNCLOUDSYNC_EXPORT requestVerb(const QNetworkReply &reply);
-
 /** Nicer errorString() for QNetworkReply
  *
  * By default QNetworkReply::errorString() often produces messages like
