@@ -17,6 +17,7 @@
 #include "clientsideencryptionjobs.h"
 #include "theme.h"
 #include "creds/abstractcredentials.h"
+#include "common/syncjournaldb.h"
 
 Q_LOGGING_CATEGORY(lcSignPublicKeyApiJob, "nextcloud.sync.networkjob.sendcsr", QtInfoMsg)
 Q_LOGGING_CATEGORY(lcStorePrivateKeyApiJob, "nextcloud.sync.networkjob.storeprivatekey", QtInfoMsg)
@@ -153,8 +154,12 @@ bool UpdateMetadataApiJob::finished()
 UnlockEncryptFolderApiJob::UnlockEncryptFolderApiJob(const AccountPtr& account,
                                                  const QByteArray& fileId,
                                                  const QByteArray& token,
+                                                 SyncJournalDb *journalDb,
                                                  QObject* parent)
-: AbstractNetworkJob(account, e2eeBaseUrl() + QStringLiteral("lock/") + fileId, parent), _fileId(fileId), _token(token)
+    : AbstractNetworkJob(account, e2eeBaseUrl() + QStringLiteral("lock/") + fileId, parent)
+    , _fileId(fileId)
+    , _token(token)
+    , _journalDb(journalDb)
 {
 }
 
@@ -169,15 +174,26 @@ void UnlockEncryptFolderApiJob::start()
 
     AbstractNetworkJob::start();
     qCInfo(lcCseJob()) << "Starting the request to unlock.";
+
+    qCInfo(lcCseJob()) << "unlock folder started for:" << path() << " for fileId: " << _fileId;
 }
 
 bool UnlockEncryptFolderApiJob::finished()
 {
     int retCode = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+
+    qCInfo(lcCseJob()) << "unlock folder finished with code" << retCode << " for:" << path() << " for fileId: " << _fileId;
+
+    if (retCode != 0) {
+        _journalDb->deleteE2EeLockedFolder(_fileId);
+    }
+
+    emit done();
+
     if (retCode != 200) {
         qCInfo(lcCseJob()) << "error unlocking file" << path() << errorString() << retCode;
         qCInfo(lcCseJob()) << "Full Error Log" << reply()->readAll();
-        emit error(_fileId, retCode);
+        emit error(_fileId, retCode, errorString());
         return true;
     }
     emit success(_fileId);
@@ -217,13 +233,33 @@ bool DeleteMetadataApiJob::finished()
     return true;
 }
 
-LockEncryptFolderApiJob::LockEncryptFolderApiJob(const AccountPtr& account, const QByteArray& fileId, QObject* parent)
-: AbstractNetworkJob(account, e2eeBaseUrl() + QStringLiteral("lock/") + fileId, parent), _fileId(fileId)
+LockEncryptFolderApiJob::LockEncryptFolderApiJob(const AccountPtr &account,
+                                                 const QByteArray &fileId,
+                                                 SyncJournalDb *journalDb,
+                                                 const QSslKey publicKey,
+                                                 QObject *parent)
+    : AbstractNetworkJob(account, e2eeBaseUrl() + QStringLiteral("lock/") + fileId, parent)
+    , _fileId(fileId)
+    , _journalDb(journalDb)
+    , _publicKey(publicKey)
 {
 }
 
 void LockEncryptFolderApiJob::start()
 {
+    const auto folderTokenEncrypted = _journalDb->e2EeLockedFolder(_fileId);
+
+    if (!folderTokenEncrypted.isEmpty()) {
+        qCInfo(lcCseJob()) << "lock folder started for:" << path() << " for fileId: " << _fileId << " but we need to first lift the previous lock";
+        const auto folderToken = EncryptionHelper::decryptStringAsymmetric(_account->e2e()->_privateKey, folderTokenEncrypted);
+        const auto unlockJob = new OCC::UnlockEncryptFolderApiJob(_account, _fileId, folderToken, _journalDb, this);
+        connect(unlockJob, &UnlockEncryptFolderApiJob::done, this, [this]() {
+            this->start();
+        });
+        unlockJob->start();
+        return;
+    }
+
     QNetworkRequest req;
     req.setRawHeader("OCS-APIREQUEST", "true");
     QUrlQuery query;
@@ -234,23 +270,32 @@ void LockEncryptFolderApiJob::start()
     qCInfo(lcCseJob()) << "locking the folder with id" << _fileId << "as encrypted";
     sendRequest("POST", url, req);
     AbstractNetworkJob::start();
+
+    qCInfo(lcCseJob()) << "lock folder started for:" << path() << " for fileId: " << _fileId;
 }
 
 bool LockEncryptFolderApiJob::finished()
 {
     int retCode = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+
     if (retCode != 200) {
         qCInfo(lcCseJob()) << "error locking file" << path() << errorString() << retCode;
-        emit error(_fileId, retCode);
+        emit error(_fileId, retCode, errorString());
+        qCInfo(lcCseJob()) << "lock folder finished with code" << retCode << " for:" << path() << " for fileId: " << _fileId;
         return true;
     }
 
     QJsonParseError error;
-    auto json = QJsonDocument::fromJson(reply()->readAll(), &error);
-    auto obj = json.object().toVariantMap();
-    auto token = obj["ocs"].toMap()["data"].toMap()["e2e-token"].toByteArray();
+    const auto json = QJsonDocument::fromJson(reply()->readAll(), &error);
+    const auto obj = json.object().toVariantMap();
+    const auto token = obj["ocs"].toMap()["data"].toMap()["e2e-token"].toByteArray();
     qCInfo(lcCseJob()) << "got json:" << token;
 
+    qCInfo(lcCseJob()) << "lock folder finished with code" << retCode << " for:" << path() << " for fileId: " << _fileId << " token:" << token;
+
+    const auto folderTokenEncrypted = EncryptionHelper::encryptStringAsymmetric(_publicKey, token);
+    _journalDb->setE2EeLockedFolder(_fileId, folderTokenEncrypted);
+
     //TODO: Parse the token and submit.
     emit success(_fileId, token);
     return true;
@@ -282,7 +327,7 @@ bool SetEncryptionFlagApiJob::finished()
         emit success(_fileId);
     } else {
         qCInfo(lcCseJob()) << "Setting the encrypted flag failed with" << path() << errorString() << retCode;
-        emit error(_fileId, retCode);
+        emit error(_fileId, retCode, errorString());
     }
     return true;
 }
