@@ -32,11 +32,20 @@ class ExcludedFiles;
 
 namespace OCC {
 
+namespace LocalDiscoveryEnums {
+
+OCSYNC_EXPORT Q_NAMESPACE
+
 enum class LocalDiscoveryStyle {
     FilesystemOnly, //< read all local data from the filesystem
     DatabaseAndFilesystem, //< read from the db, except for listed paths
 };
 
+Q_ENUM_NS(LocalDiscoveryStyle)
+
+}
+
+using OCC::LocalDiscoveryEnums::LocalDiscoveryStyle;
 
 class Account;
 class SyncJournalDb;
@@ -59,18 +68,27 @@ struct RemoteInfo
     bool isDirectory = false;
     bool isE2eEncrypted = false;
     QString e2eMangledName;
+    bool sharedByMe = false;
 
-    bool isValid() const { return !name.isNull(); }
+    [[nodiscard]] bool isValid() const { return !name.isNull(); }
 
     QString directDownloadUrl;
     QString directDownloadCookies;
+
+    SyncFileItem::LockStatus locked = SyncFileItem::LockStatus::UnlockedItem;
+    QString lockOwnerDisplayName;
+    QString lockOwnerId;
+    SyncFileItem::LockOwnerType lockOwnerType = SyncFileItem::LockOwnerType::UserLock;
+    QString lockEditorApp;
+    qint64 lockTime = 0;
+    qint64 lockTimeout = 0;
 };
 
 struct LocalInfo
 {
     /** FileName of the entry (this does not contains any directory or path, just the plain name */
     QString name;
-    QString renameName;
+    QString caseClashConflictingName;
     time_t modtime = 0;
     int64_t size = 0;
     uint64_t inode = 0;
@@ -79,7 +97,7 @@ struct LocalInfo
     bool isHidden = false;
     bool isVirtualFile = false;
     bool isSymLink = false;
-    bool isValid() const { return !name.isNull(); }
+    [[nodiscard]] bool isValid() const { return !name.isNull(); }
 };
 
 /**
@@ -95,11 +113,11 @@ public:
 
     void run() override;
 signals:
-    void finished(QVector<LocalInfo> result);
+    void finished(QVector<OCC::LocalInfo> result);
     void finishedFatalError(QString errorString);
     void finishedNonFatalError(QString errorString);
 
-    void itemDiscovered(SyncFileItemPtr item);
+    void itemDiscovered(OCC::SyncFileItemPtr item);
     void childIgnored(bool b);
 private slots:
 private:
@@ -127,9 +145,9 @@ public:
 
     // This is not actually a network job, it is just a job
 signals:
-    void firstDirectoryPermissions(RemotePermissions);
+    void firstDirectoryPermissions(OCC::RemotePermissions);
     void etag(const QByteArray &, const QDateTime &time);
-    void finished(const HttpResult<QVector<RemoteInfo>> &result);
+    void finished(const OCC::HttpResult<QVector<OCC::RemoteInfo>> &result);
 
 private slots:
     void directoryListingIteratedSlot(const QString &, const QMap<QString, QString> &);
@@ -180,6 +198,8 @@ class DiscoveryPhase : public QObject
      */
     QMap<QString, SyncFileItemPtr> _deletedItem;
 
+    QVector<QString> _directoryNamesToRestoreOnPropagation;
+
     /** Maps the db-path of a deleted folder to its queued job.
      *
      * If a folder is deleted and must be recursed into, its job isn't
@@ -211,7 +231,7 @@ class DiscoveryPhase : public QObject
      * Useful for avoiding processing of items that have already been claimed in
      * a rename (would otherwise be discovered as deletions).
      */
-    bool isRenamed(const QString &p) const { return _renamedItemsLocal.contains(p) || _renamedItemsRemote.contains(p); }
+    [[nodiscard]] bool isRenamed(const QString &p) const { return _renamedItemsLocal.contains(p) || _renamedItemsRemote.contains(p); }
 
     int _currentlyActiveJobs = 0;
 
@@ -221,7 +241,7 @@ class DiscoveryPhase : public QObject
 
     void scheduleMoreJobs();
 
-    bool isInSelectiveSyncBlackList(const QString &path) const;
+    [[nodiscard]] bool isInSelectiveSyncBlackList(const QString &path) const;
 
     // Check if the new folder should be deselected or not.
     // May be async. "Return" via the callback, true if the item is blacklisted
@@ -233,7 +253,7 @@ class DiscoveryPhase : public QObject
      * Note that it only considers parent directory renames. So if A/B got renamed to C/D,
      * checking A/B/file would yield C/D/file, but checking A/B would yield A/B.
      */
-    QString adjustRenamedPath(const QString &original, SyncFileItem::Direction) const;
+    [[nodiscard]] QString adjustRenamedPath(const QString &original, SyncFileItem::Direction) const;
 
     /** If the db-path is scheduled for deletion, abort it.
      *
@@ -248,6 +268,8 @@ class DiscoveryPhase : public QObject
      */
     QPair<bool, QByteArray> findAndCancelDeletedJob(const QString &originalPath);
 
+    void enqueueDirectoryToDelete(const QString &path, ProcessDirectoryJob* const directoryJob);
+
 public:
     // input
     QString _localDir; // absolute path to the local directory. ends with '/'
@@ -258,6 +280,7 @@ public:
     ExcludedFiles *_excludes;
     QRegularExpression _invalidFilenameRx; // FIXME: maybe move in ExcludedFiles
     QStringList _serverBlacklistedFiles; // The blacklist from the capabilities
+    QStringList _leadingAndTrailingSpacesFilesAllowed;
     bool _ignoreHiddenFiles = false;
     std::function<bool(const QString &)> _shouldDiscoverLocaly;
 
@@ -269,10 +292,14 @@ public:
     // output
     QByteArray _dataFingerprint;
     bool _anotherSyncNeeded = false;
+    QHash<QString, long long> _filesNeedingScheduledSync;
+    QVector<QString> _filesUnscheduleSync;
+
+    QStringList _listExclusiveFiles;
 
 signals:
     void fatalError(const QString &errorString);
-    void itemDiscovered(const SyncFileItemPtr &item);
+    void itemDiscovered(const OCC::SyncFileItemPtr &item);
     void finished();
 
     // A new folder was discovered and was not synced because of the confirmation feature
