@@ -30,9 +30,6 @@ class QJsonObject;
 
 namespace OCC {
 
-/** Strips quotes and gzip annotations */
-OWNCLOUDSYNC_EXPORT QByteArray parseEtag(const char *header);
-
 struct HttpError
 {
     int code; // HTTP error code
@@ -60,6 +57,31 @@ private slots:
     bool finished() override;
 };
 
+/**
+ * @brief A basic file manipulation job
+ * @ingroup libsync
+ */
+class OWNCLOUDSYNC_EXPORT SimpleFileJob : public AbstractNetworkJob
+{
+    Q_OBJECT
+public:
+    explicit SimpleFileJob(AccountPtr account, const QString &filePath, QObject *parent = nullptr);
+
+    QNetworkReply *startRequest(
+        const QByteArray &verb, const QNetworkRequest req = QNetworkRequest(), QIODevice *requestBody = nullptr);
+
+    QNetworkReply *startRequest(const QByteArray &verb, const QUrl &url, const QNetworkRequest req = QNetworkRequest(),
+        QIODevice *requestBody = nullptr);
+
+signals:
+    void finishedSignal(QNetworkReply *reply);
+protected slots:
+    bool finished() override;
+
+private:
+    QByteArray _verb;
+};
+
 /**
  * @brief sends a DELETE http request to a url.
  *
@@ -67,7 +89,7 @@ private slots:
  *
  * This does *not* delete files, it does a http request.
  */
-class OWNCLOUDSYNC_EXPORT DeleteApiJob : public AbstractNetworkJob
+class OWNCLOUDSYNC_EXPORT DeleteApiJob : public SimpleFileJob
 {
     Q_OBJECT
 public:
@@ -125,7 +147,7 @@ public:
      *    e.g. "ns:with:colons:bar", which is "bar" in the "ns:with:colons" namespace
      */
     void setProperties(QList<QByteArray> properties);
-    QList<QByteArray> properties() const;
+    [[nodiscard]] QList<QByteArray> properties() const;
 
 signals:
     void directoryListingSubfolders(const QStringList &items);
@@ -167,7 +189,7 @@ public:
      *    e.g. "ns:with:colons:bar", which is "bar" in the "ns:with:colons" namespace
      */
     void setProperties(QList<QByteArray> properties);
-    QList<QByteArray> properties() const;
+    [[nodiscard]] QList<QByteArray> properties() const;
 
 signals:
     void result(const QVariantMap &values);
@@ -243,7 +265,7 @@ public:
      *    e.g. "ns:with:colons:bar", which is "bar" in the "ns:with:colons" namespace
      */
     void setProperties(QMap<QByteArray, QByteArray> properties);
-    QMap<QByteArray, QByteArray> properties() const;
+    [[nodiscard]] QMap<QByteArray, QByteArray> properties() const;
 
 signals:
     void success();
@@ -357,6 +379,58 @@ private slots:
     bool finished() override;
 };
 
+class OWNCLOUDSYNC_EXPORT SimpleApiJob : public AbstractNetworkJob
+{
+    Q_OBJECT
+public:
+    enum class Verb {
+        Get,
+        Post,
+        Put,
+        Delete,
+        };
+
+    explicit SimpleApiJob(const AccountPtr &account, const QString &path, QObject *parent = nullptr);
+
+    void setBody(const QByteArray &body);
+
+    void setVerb(Verb value);
+
+    /**
+     * @brief addQueryParams - add more parameters to the ocs call
+     * @param params: list pairs of strings containing the parameter name and the value.
+     *
+     * All parameters from the passed list are appended to the query. Note
+     * that the format=json parameter is added automatically and does not
+     * need to be set this way.
+     *
+     * This function needs to be called before start() obviously.
+     */
+    void addQueryParams(const QUrlQuery &params);
+    void addRawHeader(const QByteArray &headerName, const QByteArray &value);
+
+public slots:
+    void start() override;
+
+Q_SIGNALS:
+
+    void resultReceived(int statusCode);
+
+protected:
+    bool finished() override;
+
+    [[nodiscard]] QNetworkRequest& request();
+    [[nodiscard]] QByteArray& body();
+    [[nodiscard]] QUrlQuery& additionalParams();
+    [[nodiscard]] QByteArray verbToString() const;
+
+private:
+    QByteArray _body;
+    QUrlQuery _additionalParams;
+    QNetworkRequest _request;
+    Verb _verb = Verb::Get;
+};
+
 /**
  * @brief Job to check an API that return JSON
  *
@@ -372,36 +446,14 @@ private slots:
  *
  * @ingroup libsync
  */
-class OWNCLOUDSYNC_EXPORT JsonApiJob : public AbstractNetworkJob
+class OWNCLOUDSYNC_EXPORT JsonApiJob : public SimpleApiJob
 {
     Q_OBJECT
 public:
-    enum class Verb {
-        Get,
-        Post,
-        Put,
-        Delete,
-    };
-
     explicit JsonApiJob(const AccountPtr &account, const QString &path, QObject *parent = nullptr);
 
-    /**
-     * @brief addQueryParams - add more parameters to the ocs call
-     * @param params: list pairs of strings containing the parameter name and the value.
-     *
-     * All parameters from the passed list are appended to the query. Note
-     * that the format=json parameter is added automatically and does not
-     * need to be set this way.
-     *
-     * This function needs to be called before start() obviously.
-     */
-    void addQueryParams(const QUrlQuery &params);
-    void addRawHeader(const QByteArray &headerName, const QByteArray &value);
-
     void setBody(const QJsonDocument &body);
 
-    void setVerb(Verb value);
-
 public slots:
     void start() override;
 
@@ -423,21 +475,6 @@ signals:
      * @param statusCode - the OCS status code: 100 (!) for success
      */
     void etagResponseHeaderReceived(const QByteArray &value, int statusCode);
-
-    /**
-     * @brief desktopNotificationStatusReceived - signal to report if notifications are allowed
-     * @param status - set desktop notifications allowed status
-     */
-    void allowDesktopNotificationsChanged(bool isAllowed);
-
-private:
-    QByteArray _body;
-    QUrlQuery _additionalParams;
-    QNetworkRequest _request;
-
-    Verb _verb = Verb::Get;
-
-    QByteArray verbToString() const;
 };
 
 /**
@@ -462,7 +499,7 @@ public:
     explicit DetermineAuthTypeJob(AccountPtr account, QObject *parent = nullptr);
     void start();
 signals:
-    void authType(AuthType);
+    void authType(OCC::DetermineAuthTypeJob::AuthType);
 
 private:
     void checkAllDone();
